/*
 *  (c) 2004 Grigory Batalov <bga@altlinux.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  Test program for NDiags-type LPT plug.
 *
 *  Plug's pinout:
 *  2 <-> 15	(Data0 -> Error)
 *  3 <-> 13	(Data1 -> Offline)
 *  4 <-> 12	(Data2 -> No Paper)
 *  5 <-> 10	(Data3 -> Acknowledge)
 *  6 <-> 11	(Data4 -> Busy)
 *  (5 low bits of Data go to 5 high bits of Status)
 *
 *  Compile under DOS with Turbo C 2.0:
 *   tcc -Ic:\tc20 -Lc:\tc20 -DDOS port.c
 *
 *  Compile under Linux with GCC:
 *   gcc -g -Wall -O2 -DLINUX -o port port.c
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef LINUX
#	include <unistd.h>
#	include <sys/io.h>
#	define OUTB(a,b) outb(a,b)
#	define INB(a) inb(a)
#else
#	ifdef DOS
#		include <dos.h>
#		define OUTB(a,b) outportb(b,a)
#		define INB(a) inportb(a)
#	else
#		error "Define or -DLINUX either -DDOS"
#	endif
#endif

int test_byte(int base, int byte) {
	int before,after;
	/* send zero */
	OUTB(0, base);
	OUTB(1, base + 2);
	OUTB(0, base + 2);
	/* check status */
	before = INB(base + 1);

	/* send byte */
	OUTB(byte, base);
	OUTB(1, base+2);
	OUTB(0, base+2);
	/* get new status */
	after = INB(base + 1);

	/* calculate difference in status
	   as lower byte of (a XOR b) */
	return (after^before) & 0xff;
}

int main(int argc, char* argv[]) {
    int i,j, base, diff1,diff2, do_debug = 0;

    /* names for status bits */
    char *status[] = {	"Error",	/* bit 3 */
			"Offline",	/* bit 4 */
			"No Paper",	/* bit 5 */
			"Acknowledge",	/* bit 6 */
			"Busy" };	/* bit 7 */

    if (argc < 2 || (base = (int) strtol(argv[1], NULL, 0)) == 0) {
	printf ("Usage:\n %s <port> [-d]\n"
		" <port>\t\t - port number, example: 0x378\n"
		" -d\t\t - show debug messages\n",
		argv[0]);
	exit(1);
    }

    printf("Using port 0x%03x\n",base);

    if (argc > 2 && !strncmp(argv[2], "-d", 2))
	do_debug = 1;

#ifdef LINUX
    if (ioperm(base, 3, 1)) {
	perror("ioperm");
	exit(1);
    }
#endif

    for (i = 0; i < 5; i++) {
	printf ("Testing for '%s' (0x%02x)\n", status[i], 1<<i);

	for (j = 0; j < 1<<5; j++) {
	    /* test enabled bit */
	    diff1 = test_byte(base, j | (1<<i));

	    /* test disabled bit */
	    diff2 = test_byte(base, j & ~(1<<i));

	    if (do_debug)
		printf ("%02d) Sent 0x%02x, got 0x%02x; sent 0x%02x, got 0x%02x; difference: 0x%02x\n",
			j, j | (1<<i), diff1, j & ~(1<<i), diff2, diff1^diff2);

	    /* should differ in only status bit */
	    if ((diff1^diff2) != 1<<(i+3)) {
		printf ("Signal '%s' failed (sent 0x%02x, got 0x%02x, expected 0x%02x)\n",
			status[i], j | (1<<i), diff1, 1<<(i+3));
		exit(1);
	    }
	}
    }
    printf ("All tests passed\n");
    exit(0);
}




